@@ -27,10 +27,10 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#include <glib/gi18n.h>
 #include <ORBitservices/CosNaming.h>
 #include <ORBitservices/CosNaming_impl.h>
 #include <libbonobo.h>
-#include <bonobo/bonobo-i18n.h>
 
 #include "bonobo-activation/bonobo-activation-private.h"
 
@@ -54,6 +54,10 @@
 
 #include <glib/gstdio.h>
 
+#ifdef HAVE_DBUS
+#include <dbus/dbus-glib-lowlevel.h>
+#endif
+
 static gboolean        server_threaded = FALSE;
 static glong           server_guard_depth = 0;
 static GStaticRecMutex server_guard = G_STATIC_REC_MUTEX_INIT;
@@ -183,6 +187,7 @@ static const GOptionEntry options[] = {
         {NULL}
 };
 
+/* Referenced from object-directory-corba.c */
 GMainLoop *main_loop = NULL;
 
 static GString *
@@ -191,7 +196,6 @@ build_src_dir (void)
         const char *env_od_source_dir;
         const char *gnome_env_od_source_dir;
         char *config_file_od_source_dir;
-        GString *gnome_od_source_dir;
         char **gnome_dirs;
         GString *real_od_source_dir;
         int i;
@@ -220,6 +224,7 @@ build_src_dir (void)
         }
 
         if (gnome_env_od_source_dir) {
+                GString *gnome_od_source_dir;
                 gnome_dirs = g_strsplit (gnome_env_od_source_dir, G_SEARCHPATH_SEPARATOR_S, -1);
                 gnome_od_source_dir = g_string_new("");
                 for (i=0; gnome_dirs[i]; i++) {
@@ -232,6 +237,7 @@ build_src_dir (void)
                 g_string_append (real_od_source_dir,
                                  gnome_od_source_dir->str);
                 g_string_append_c (real_od_source_dir, G_SEARCHPATH_SEPARATOR);
+		g_string_free (gnome_od_source_dir, TRUE);
         }
 
         g_string_append (real_od_source_dir, SERVERINFODIR);
@@ -362,9 +368,7 @@ dump_ior (CORBA_ORB orb, int dev_null_fd, CORBA_Environment *ev)
 #endif
         if (server_reg) {
                 char *fname;
-                fname = g_build_filename (ORBit_get_safe_tmp (),
-                                          "bonobo-activation-server-ior",
-                                          NULL);
+                fname = _bonobo_activation_ior_fname_get ();
                 fh = g_fopen (fname, "w+");
 		fprintf (fh, "%s\n", ior);
 		fclose (fh);
@@ -374,6 +378,39 @@ dump_ior (CORBA_ORB orb, int dev_null_fd, CORBA_Environment *ev)
 	CORBA_free (ior);
 }
 
+static void
+cleanup_ior_and_lock_files (void)
+{
+    char *fname;
+
+    fname = _bonobo_activation_ior_fname_get ();
+    g_unlink (fname);
+    g_free (fname);
+
+    fname = _bonobo_activation_lock_fname_get ();
+    g_unlink (fname);
+    g_free (fname);
+}
+
+#ifdef HAVE_DBUS
+
+static DBusHandlerResult
+bus_message_handler (DBusConnection *connection,
+                     DBusMessage    *message,
+                     GMainLoop      *loop)
+{
+        if (dbus_message_is_signal (message,
+                                    DBUS_INTERFACE_LOCAL,
+                                    "Disconnected")) {
+                g_main_loop_quit (loop);
+                return DBUS_HANDLER_RESULT_HANDLED;
+        }
+
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -397,6 +434,10 @@ main (int argc, char *argv[])
 	const gchar                  *debug_output_env;
 #endif
 	GError                       *error = NULL;
+#ifdef HAVE_DBUS
+        DBusConnection               *connection;
+        DBusError                     bus_error;
+#endif
 
 #ifdef HAVE_SETSID
         /*
@@ -534,6 +575,28 @@ main (int argc, char *argv[])
 
 	od_finished_internal_registration (); 
 
+#ifdef HAVE_DBUS
+        dbus_error_init (&bus_error);
+        connection = dbus_bus_get (DBUS_BUS_SESSION, &bus_error);
+
+        if (dbus_error_is_set (&bus_error)) {
+                g_warning ("could not associate with desktop session: %s",
+                           bus_error.message);
+                connection = NULL;
+        } else {
+                GMainContext *main_context;
+
+                main_context = g_main_loop_get_context (main_loop);
+                dbus_connection_setup_with_g_main (connection, main_context);
+
+                if (dbus_connection_add_filter (connection,
+                                                (DBusHandleMessageFunction)
+                                                bus_message_handler, main_loop,
+                                                NULL)) {
+                        dbus_connection_set_exit_on_disconnect (connection, FALSE);
+                }
+        }
+#endif
         if (getenv ("BONOBO_ACTIVATION_DEBUG") == NULL)
                 chdir ("/");
 
@@ -549,6 +612,8 @@ main (int argc, char *argv[])
         bonobo_object_directory_shutdown (threaded_poa, ev);
         activation_context_shutdown ();
 
+        cleanup_ior_and_lock_files ();
+
 #ifdef HAVE_SYSLOG_H
 	closelog ();
 	g_free (syslog_ident);
